<!doctype html>
<!-- Current Week Calendar display for iPads or Ubiquiti Connect Display in vertical mode -->
<!-- By Eric Bylenga -->
<!-- Last updated: December 20, 2024 -->
<!-- https://bylenga.ddns.net
<!-- Version 2.0 -->
<html>
<head>
<meta charset="UTF-8">
<meta name="apple-mobile-web-app-capable" content="yes">
<title>Calendar</title>
	
	<!-- Feel Free to Modify this CSS code to fit your organization's colour schemes etc... -->
	<style>
		body {
			font-family: Arial, "sans-serif";
			background-image: url("happymacfade.webp");
			background-repeat: repeat;
			background-attachment: fixed;
			font-size:2.0rem;
		}
		table {
			width:100%;
		}
		#daySeparator {
			background-color:rgba(182, 208, 226, 0.85);
			font-size:1.25em;
		}
		#alternateRow {
			background-color:rgba(229, 228, 226, 0.85);
		}
		#eventHighlight {
			background-color:rgba(250, 160, 160, 0.9);
		}
	</style>
	<!--------------------------End Modify CSS Code----------------------------------------->
	
	<script src="https://ajax.googleapis.com/ajax/libs/jquery/3.6.1/jquery.min.js"></script>
	<script>
		//Detect if connection online (define a file to check for on the server to see if the server is up. Otherwise don't refresh the screen... It'll just make things messy.)
		setInterval(function() {
		$.ajax({ 
            url  : "calendar/happymac.webp", // or other resource... This is the file we're checking for, but you can change this.
            type : "HEAD"
        })
        .done(function() {
            location.reload();
        });
}, 300000); // 300000 ~> 5 minutes refresh. This can be adjusted of course.
		
		
		//Show and Hide List of Calendars
		function showHide(thing) {
  		var x = document.getElementById(thing);
  		if (x.style.display === "none") {
    		x.style.display = "block";
  		} else {
    		x.style.display = "none";
  		}
	}
	</script>
</head>
<body>
<?php
//--------------------------------------- Variables to Change --------------------------------------------//
  
	//Calendar Array -  First Value is a name that will be applied to the top of the display. Second value in the line is the public iCal URL we wish to pull from. Add more lines as required.
    $calendarurls = array(
        array("Conference Room","https://calendar.google.com/calendar/ical/testcalendar.calendar.google.com/public/basic.ics"),
        array("The Gym","https://calendar.google.com/calendar/ical/secondcalendar.calendar.google.com/public/basic.ics")
        

    );
	
	//Keyword images... For example, if a calendar event contains the word 'Computer', the background will change to the computer.png background. For ipads I'm using a size of about 1000x1333 or so for a single image to appear without repeating. Light grayscales against a white background seem to work well. The image will repeat on x and y axis unless you modify the css code at the top of this page.
	$imageMap = [
        'Graphic'    => 'calendar/graphic3.png',
        'Coffee'     => 'calendar/coffee.png',
        'Basketball' => 'calendar/basketball.png',
        'Volleyball' => 'calendar/volleyball.png',
        'Robot'      => 'calendar/robot.png',
        'Lunch'      => 'calendar/lunch.png',
		'Meeting'	 => 'calendar/meeting.png',
		'Mtg'	 	 => 'calendar/meeting.png',
    ];

//Default background filename (100x100px is a good default)
$backgroundImage = 'calendar/happymacfade.webp';

//Icon next to Calendar name (like your company logo or whatever. I'm using 72x72px)
$icon = 'calendar/happymac.webp';
	
//Set your timezone
$localTimeZone = "America/Vancouver";
	
//--------------------------------------- End Variables to Change -------------------------------------//
	
require 'vendor/autoload.php';

use Sabre\VObject\Reader;
use Sabre\VObject\Recur;
	
	
if(isset($_GET['title']))
	{
		$urlkey = array_search(strip_tags(htmlentities($_GET['title'])),array_column($calendarurls,0));
		$icalUrl = $calendarurls[$urlkey][1];
		$CalendarTitle = strip_tags(htmlentities($_GET['title']));
	}
	else{
		$icalUrl = $calendarurls[0][1];
		$CalendarTitle = $calendarurls[0][0];
	}

	echo "<span style='font-size:1.5em;'><img src='". $icon ."' style='height:1.5em;'><strong>". $CalendarTitle ."</strong></span><br>";

$icalData = file_get_contents($icalUrl);

// Parse the iCal data
$vcalendar = Reader::read($icalData);

// Set the default timezone to UTC for consistent comparisons
date_default_timezone_set('UTC');

// Define the time range for events (next 7 days)
$now = new DateTimeImmutable('now', new DateTimeZone('UTC'));
$oneWeekLater = new DateTimeImmutable('+7 days', new DateTimeZone('UTC'));

$events = [];
$processedEvents = []; // To avoid duplicates

foreach ($vcalendar->VEVENT as $vevent) {
    // Get the UID
    $uid = (string)$vevent->UID;

    // Handle EXDATEs (Excluded Dates)
    $exdates = [];
    if (isset($vevent->EXDATE)) {
        foreach ($vevent->EXDATE as $exdate) {
            // Retrieve DateTimeImmutable objects with correct time zones
            foreach ($exdate->getDateTimes() as $exDateTime) {
                // Convert to UTC for comparison
                $exDateTimeUTC = $exDateTime->setTimeZone(new DateTimeZone('UTC'));
                $exdates[] = $exDateTimeUTC;
            }
        }
    }

    // Handle Recurring Events
    if (isset($vevent->RRULE)) {
        // Ensure DTSTART exists
        if (!isset($vevent->DTSTART)) {
            error_log("Event UID $uid is missing DTSTART.");
            continue;
        }

        try {
            $rruleIterator = new Recur\EventIterator($vcalendar, $uid);
        } catch (Recur\NoInstancesException $e) {
            error_log("NoInstancesException for event UID: $uid");
            continue;
        } catch (\Exception $e) {
            error_log("Exception for event UID $uid: " . $e->getMessage());
            continue;
        }

        // Iterate over occurrences
        while ($rruleIterator->valid()) {
            // Get occurrence start and end times
            $occurrenceStart = $rruleIterator->getDTStart();
            $occurrenceEnd = $rruleIterator->getDTEnd();

            // Convert occurrence times to UTC for comparison
            $occurrenceStartUTC = $occurrenceStart->setTimeZone(new DateTimeZone('UTC'));
            $occurrenceEndUTC = $occurrenceEnd->setTimeZone(new DateTimeZone('UTC'));

            // Check if the occurrence is within the next 7 days
            if ($occurrenceStartUTC > $oneWeekLater) {
                break; // Beyond our time range
            }

            if ($occurrenceEndUTC < $now) {
                $rruleIterator->next();
                continue; // Skip past events
            }

            // Check for Excluded Dates
            $isExcluded = false;
            foreach ($exdates as $exdateUTC) {
                if ($occurrenceStartUTC == $exdateUTC) {
                    $isExcluded = true;
                    break;
                }
            }
            if ($isExcluded) {
                $rruleIterator->next();
                continue;
            }

            // Exclude Declined Events (PARTSTAT)
            $includeEvent = true;
            if (isset($vevent->ATTENDEE)) {
                foreach ($vevent->ATTENDEE as $attendee) {
                    if (isset($attendee['PARTSTAT']) && strtoupper($attendee['PARTSTAT']->getValue()) === 'DECLINED') {
                        $includeEvent = false;
                        break;
                    }
                }
            }

            if ($includeEvent) {
                // Handle Multiday Events
                $startDate = $occurrenceStartUTC->setTime(0, 0, 0);
                $endDate = $occurrenceEndUTC->setTime(0, 0, 0);

                // Include the last day in the period
                $periodEndDate = $endDate->add(new DateInterval('P1D'));

                $interval = new DateInterval('P1D');
                $period = new DatePeriod($startDate, $interval, $periodEndDate);

                foreach ($period as $date) {
                    // Adjust date to local time zone
                    $eventDateLocal = $date->setTimeZone(new DateTimeZone($localTimeZone));

                    // Set start and end times for each day's event
                    if ($date->format('Y-m-d') == $occurrenceStartUTC->format('Y-m-d')) {
                        // First day: use actual occurrence start time
                        $dayStartLocal = $occurrenceStart->setTimeZone(new DateTimeZone($localTimeZone));
                    } else {
                        // Middle days: start at 00:00:00 local time
                        $dayStartLocal = $eventDateLocal->setTime(0, 0, 0);
                    }

                    if ($date->format('Y-m-d') == $occurrenceEndUTC->format('Y-m-d')) {
                        // Last day: use actual occurrence end time
                        $dayEndLocal = $occurrenceEnd->setTimeZone(new DateTimeZone($localTimeZone));
                    } else {
                        // Middle days: end at 23:59:59 local time
                        $dayEndLocal = $eventDateLocal->setTime(23, 59, 59);
                    }

                    // Convert dayStartLocal back to UTC for comparison
                    $dayStartUTC = $dayStartLocal->setTimeZone(new DateTimeZone('UTC'));
					$dayEndUTC = $dayEndLocal->setTimeZone(new DateTimeZone('UTC'));

                    if ($dayEndLocal->setTimeZone(new DateTimeZone('UTC')) < $now) {
                        continue; // Skip past days
                    }

                    if ($dayEndUTC >= $now && $dayStartUTC <= $oneWeekLater) {
                        // Unique key to avoid duplicates
                        $eventKey = $uid . '|' . $dayStartLocal->format('Y-m-d');

                        if (!isset($processedEvents[$eventKey])) {
                            $processedEvents[$eventKey] = true;

                            $events[] = [
                                'uid' => $uid,
                                'summary' => (string)$vevent->SUMMARY,
                                'start' => $dayStartLocal->format('Y-m-d H:i:s'),
                                'end' => $dayEndLocal->format('Y-m-d H:i:s'),
                                'description' => isset($vevent->DESCRIPTION) ? (string)$vevent->DESCRIPTION : '',
                            ];
                        }
                    }
                }
            }

            $rruleIterator->next();
        }
    } else {
        // Non-recurring event
        if (!isset($vevent->DTSTART)) {
            error_log("Event UID $uid is missing DTSTART.");
            continue;
        }

        // Get start and end times correctly
        $dtstart = getDateTimeInUTC($vevent->DTSTART);
        $dtend = isset($vevent->DTEND) ? getDateTimeInUTC($vevent->DTEND) : null;

        if ($dtend && $dtend < $now) {
            continue; // Skip past events
        }

        if ($dtstart <= $oneWeekLater && ($dtend === null || $dtend >= $now)) {
            // Exclude Declined Events
            $includeEvent = true;
            if (isset($vevent->ATTENDEE)) {
                foreach ($vevent->ATTENDEE as $attendee) {
                    if (isset($attendee['PARTSTAT']) && strtoupper($attendee['PARTSTAT']->getValue()) === 'DECLINED') {
                        $includeEvent = false;
                        break;
                    }
                }
            }

            if ($includeEvent) {
                // Handle Multiday Events
                $startDate = $dtstart->setTime(0, 0, 0);
                $endDate = $dtend ? $dtend->setTime(0, 0, 0) : $startDate;

                // Include the last day in the period
                $periodEndDate = $endDate->add(new DateInterval('P1D'));

                $interval = new DateInterval('P1D');
                $period = new DatePeriod($startDate, $interval, $periodEndDate);

                foreach ($period as $date) {
                    // Adjust date to local time zone
                    $eventDateLocal = $date->setTimeZone(new DateTimeZone($localTimeZone));

                    // Set start and end times for each day's event
                    if ($date->format('Y-m-d') == $dtstart->format('Y-m-d')) {
                        // First day: use actual start time
                        $dayStartLocal = $dtstart->setTimeZone(new DateTimeZone($localTimeZone));
                    } else {
                        // Middle days: start at 00:00:00 local time
                        //$dayStartLocal = $eventDateLocal->setTime(0, 0, 0);
                        $dayStartLocal = $dtstart->setTimeZone(new DateTimeZone($localTimeZone));

                    }

                    if ($dtend && $date->format('Y-m-d') == $dtend->format('Y-m-d')) {
                        // Last day: use actual end time
                        $dayEndLocal = $dtend->setTimeZone(new DateTimeZone($localTimeZone));
                    } else {
                        // Middle days: end at 23:59:59 local time
                        //$dayEndLocal = $eventDateLocal->setTime(23, 59, 59);
						$dayEndLocal = $dtend->setTimeZone(new DateTimeZone($localTimeZone));

                    }

                    // Convert dayStartLocal back to UTC for comparison
                    $dayStartUTC = $dayStartLocal->setTimeZone(new DateTimeZone('UTC'));
					$dayEndUTC = $dayEndLocal->setTimeZone(new DateTimeZone('UTC'));


                    if ($dayEndLocal->setTimeZone(new DateTimeZone('UTC')) < $now) {
                        continue; // Skip past days
                    }

                    if ($dayEndUTC >= $now && $dayStartUTC <= $oneWeekLater) {
                        // Unique key to avoid duplicates
                        $eventKey = $uid . '|' . $dayStartLocal->format('Y-m-d');

                        if (!isset($processedEvents[$eventKey])) {
                            $processedEvents[$eventKey] = true;

                            $events[] = [
                                'uid' => $uid,
                                'summary' => (string)$vevent->SUMMARY,
                                'start' => $dayStartLocal->format('Y-m-d H:i:s'),
                                'end' => $dayEndLocal->format('Y-m-d H:i:s'),
                                'description' => isset($vevent->DESCRIPTION) ? (string)$vevent->DESCRIPTION : '',
                            ];
                        }
                    }
                }
            }
        }
    }
}

// Sort events by start time
usort($events, function ($a, $b) {
    return strtotime($a['start']) - strtotime($b['start']);
});	
	
//--------------Display our Events -------------------//
//Time to display our data
	echo "<table>";
	$dayFlag = "yo";
	$highlightFlag = false;
	date_default_timezone_set($localTimeZone);
	foreach($events as $item)
	{
		//cast start and end times to proper integers
		$starttime = strtotime($item['start']);
		$endtime = strtotime($item['end']);
		//Put a fancy Header on Each Day
		if($dayFlag != date('D',$starttime))
		{
			echo "<tr id='daySeparator'><td colspan=2><strong>". date('D M j',$starttime) . "</strong></td></tr>";
			$dayFlag = date('D',$starttime);
			$highlightFlag = false;
		}
		$meetingFlag = false;
		if (time() > $starttime && time() < $endtime)
		{
			$meetingFlag = true;
			//Change the background based on certain criterea...
            $summary = $item['summary'] ?? ''; // Ensure $summary is set

            $backgroundImage = 'calendar/happymacfade.webp'; // A fallback image if none match

            foreach ($imageMap as $term => $image) {
                if (stripos($summary, $term) !== false) {
                    $backgroundImage = $image;
                    break; // Stop on the first match
                }
            }
		}
		if($highlightFlag == false && $meetingFlag == false){echo "<tr>";$highlightFlag=true;}
		elseif($meetingFlag == true){echo "<tr id='eventHighlight'>";$meetingFlag=false;}
		else{echo "<tr id='alternateRow'>";$highlightFlag=false;}
		
		echo "<td style='width:30%;'>" . date("g:i a",$starttime) . "-". date("g:i a",$endtime) . "</td>";
		echo "<td><strong>" . $item['summary'] . "</strong>";
		echo "</td></tr>";
	}
	echo "</table>";
	
	
	
	echo '<span onclick="showHide(\'chooser\')" style="font-size:.25em;">&Pi;</span>';//Did anyone see "The Net" with Sandra Bullock?
	echo "<div id='chooser' style='display:none;'>";
	echo "<form action='calendar.php'>";
		echo "<p>";
		echo "<label for='title'>Choose a Calendar: </label>";
		echo "<select id='title' name='title'>";
		foreach($calendarurls as $title)
		{
			echo "<option value='" .$title[0] . "'>". $title[0] ."</option>" ;
		}
		echo "</select>";
		echo "<input type='submit' title='Select'>";
	echo "</form></p>";
	echo "</div>";
	
//--------------End Display our Events----------------//
	

//Function to get DateTime in UTC from a VObject Property
function getDateTimeInUTC($property) {
    $dateTime = null;

    if (isset($property['TZID'])) {
        // Has TZID parameter
        $tzid = (string)$property['TZID'];
        $dateTime = $property->getDateTime();
        $dateTimeUTC = $dateTime->setTimeZone(new DateTimeZone('UTC'));
    } else {
        $value = $property->getValue();
        if (substr($value, -1) === 'Z') {
            // Time is in UTC
            $dateTimeUTC = $property->getDateTime(new DateTimeZone('UTC'));
        } else {
            // Time is floating, assume default time zone
            $dateTime = $property->getDateTime();
            // Convert to UTC
            $dateTimeUTC = $dateTime->setTimeZone(new DateTimeZone('UTC'));
        }
    }

    return $dateTimeUTC;
}
?>
<script>
// If the chosen background is not the default, update it dynamically
// This executes in the browser, after HTML is rendered.
document.body.style.backgroundImage = "url('<?php echo $backgroundImage; ?>')";
</script>
</body>
</html>